@@ -45,6 +45,37 @@ Glib::ustring Document::get_file_uri() const
   return m_file_uri;
 }
 
+Glib::ustring Document::get_file_uri_with_extension(const Glib::ustring& uri)
+{
+  Glib::ustring result = uri;
+
+  //Enforce file extension:
+  if(!m_file_extension.empty())  //If there is an extension to enforce.
+  {
+    bool bAddExt = false;
+    Glib::ustring strExt = "." + get_file_extension();
+
+    if(result.size() < strExt.size()) //It can't have the ext already if it's not long enough.
+    {
+      bAddExt = true; //It isn't there already.
+    }
+    else
+    {
+      Glib::ustring strEnd = result.substr(result.size() - strExt.size());
+      if(strEnd != strExt) //If it doesn't already have the extension
+        bAddExt = true;
+    }
+
+    //Add extension if necessay.
+    if(bAddExt)
+      result += strExt;
+
+    //Note that this does not replace existing extensions, so it could be e.g. 'something.blah.theext'
+  }
+
+  return result;
+}
+
 void Document::set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExtension /* = false */)
 {
   if(file_uri != m_file_uri)
@@ -52,32 +83,9 @@ void Document::set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExte
 
   m_file_uri = file_uri;
 
-  //Enfore file extension:
+  //Enforce file extension:
   if(bEnforceFileExtension)
-  {
-    if(m_strFileExtension.size() != 0)  //If there is an extension to enforce.
-    {
-      bool bAddExt = false;
-      Glib::ustring strExt = "." + get_file_extension();
-      
-      if(m_file_uri.size() < strExt.size()) //It can't have the ext already if it's not long enough.
-      {
-        bAddExt = true; //It isn't there already.
-      }
-      else
-      {
-        Glib::ustring strEnd = m_file_uri.substr(m_file_uri.size()-strExt.size());
-        if(strEnd != strExt) //If it doesn't already have the extension
-          bAddExt = true;
-      }
- 
-      //Add extension if necessay.
-      if(bAddExt)
-        m_file_uri += strExt;
-        
-      //Note that this does not replace existing extensions, so it could be e.g. 'something.blah.theext'
-    }
-  }
+    m_file_uri = get_file_uri_with_extension(m_file_uri);
 }
 
 void Document::set_contents(const Glib::ustring& strVal)
@@ -97,8 +105,10 @@ void Document::set_modified(bool bVal /* = true */)
   if(m_bModified)
   {
     m_bIsNew = false; //Can't be new if it's been modified.
-    signal_modified().emit();
-  } 
+  }
+
+  //Allow the application or view to update it's UI accordingly:
+  signal_modified().emit(m_bModified);
 }
 
 bool Document::get_modified() const
@@ -123,6 +133,26 @@ bool Document::load()
   return bTest;
 }
 
+bool Document::load_from_data(const guchar* data, std::size_t length)
+{
+  if(!data || !length)
+    return false;
+
+  m_strContents = Glib::ustring((char*)data, length);
+ 
+  const bool bTest = load_after(); //may be overridden.
+  if(bTest)
+  {
+    //Tell the View to show the new data:
+    if(m_pView)
+      m_pView->load_from_document();
+  }
+
+  return bTest;
+}
+
+
+
 bool Document::load_after()
 {
   //Called after text is read from disk, but before updating view.
@@ -163,6 +193,7 @@ bool Document::read_from_disk()
   // open the input file for read access
   Gnome::Vfs::Handle read_handle;
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     read_handle.open(m_file_uri, Gnome::Vfs::OPEN_READ);
@@ -172,18 +203,31 @@ bool Document::read_from_disk()
     // If the operation was not successful, print the error and abort
     return false; //print_error(ex, input_uri_string);
   }
+#else
+  std::auto_ptr<Gnome::Vfs::exception> error;
+  read_handle.open(m_file_uri, Gnome::Vfs::OPEN_READ, error);
+  if(error.get() != NULL)
+    return false; //print_error(ex, input_uri_string);
+#endif
 
   // Read data from the input uri:
   guint buffer[BYTES_TO_PROCESS] = {0, }; // For each chunk.
   Gnome::Vfs::FileSize bytes_read = 0;
   std::string data; //We use a std::string because we might not get whole UTF8 characters at a time. This might not be necessary.
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
+#endif
   {
     bool bContinue = true;
     while(bContinue)
     {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       bytes_read = read_handle.read(buffer, BYTES_TO_PROCESS);
+#else
+      bytes_read = read_handle.read(buffer, BYTES_TO_PROCESS, error);
+      if(error.get() != NULL) break;
+#endif
 
       if(bytes_read == 0)
         bContinue = false; //stop because we reached the end.
@@ -194,8 +238,14 @@ bool Document::read_from_disk()
       }
     }
   }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   catch(const Gnome::Vfs::exception& ex)
   {
+#else
+  if(error.get() != NULL)
+  {
+    Gnome::Vfs::exception& ex = *error.get();
+#endif
     // If the operation was not successful, print the error and abort
     return false; //print_error(ex, input_uri_string);
   }
@@ -217,17 +267,28 @@ bool Document::write_to_disk()
     the second to last tells GnomeVFS that its ok if the file already exists, and just open it */
 
     Gnome::Vfs::Handle write_handle;
-      
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
-      write_handle.create(m_file_uri, Gnome::Vfs::OPEN_WRITE, false, 0x777);
+      //0660 means "this user and his group can read and write this non-executable file".
+      //The 0 prefix means that this is octal.
+      write_handle.create(m_file_uri, Gnome::Vfs::OPEN_WRITE, false, 0660 /* leading zero means octal */);
     }
     catch(const Gnome::Vfs::exception& ex)
     {
+#else
+    std::auto_ptr<Gnome::Vfs::exception> error;
+    write_handle.create(m_file_uri, Gnome::Vfs::OPEN_WRITE, false, 0660 /* loading zero means octal */, error);
+    if(error.get() != NULL)
+    {
+      const Gnome::Vfs::exception& ex = *error.get();
+#endif
      // If the operation was not successful, print the error and abort
      return false; // print_error(ex, output_uri_string);
     }
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
       //Write the data to the output uri
@@ -235,10 +296,16 @@ bool Document::write_to_disk()
     }
     catch(const Gnome::Vfs::exception& ex)
     {
+#else
+    GnomeVFSFileSize bytes_written = write_handle.write(m_strContents.data(), m_strContents.bytes(), error);
+    if(error.get() != NULL)
+    {
+      Gnome::Vfs::exception& ex = *error.get();
+#endif
       // If the operation was not successful, print the error and abort
       return false; //print_error(ex, output_uri_string);
     }
-  
+
     return true; //Success. (At doing nothing, because nothing needed to be done.)
   }
   else
@@ -247,32 +314,32 @@ bool Document::write_to_disk()
 
 Glib::ustring Document::get_name() const
 {
-  return util_file_uri_get_name(m_file_uri);
+  return util_file_uri_get_name(m_file_uri, m_file_extension);
 }
 
-Glib::ustring Document::util_file_uri_get_name(const Glib::ustring& file_uri)
+Glib::ustring Document::util_file_uri_get_name(const Glib::ustring& file_uri, const Glib::ustring& file_extension)
 {
-  Glib::ustring strResult;
+  Glib::ustring strResult = Glib::filename_display_basename(file_uri);
 
-  if(file_uri.size())
+  //Remove the file extension:
+  //TODO: Maybe filename_display_basename() should do this.
+  if(!strResult.empty() && !file_extension.empty())
   {
-    //Find position of last '/' character.
-    Glib::ustring::size_type posLastSlash = file_uri.find_last_of("/");
-    if(posLastSlash == Glib::ustring::npos)
+    const Glib::ustring strExt = "." + file_extension;
+
+    if(strResult.size() >= file_extension.size()) //It can't have the ext already if it's not long enough.
     {
-      //No '/' found. The file must be in the current or top-level directory.
-      strResult = file_uri;
-    }
-    else
-    {
-      //'/' found. The filename is the text after this: e.g. sub/sub/filename:
-      strResult = file_uri.substr(posLastSlash+1);
+      Glib::ustring strEnd = strResult.substr(strResult.size() - strExt.size());
+      if(strEnd == strExt) //If it has the extension
+      {
+        strResult = strResult.substr(0, strResult.size() - strExt.size());
+      }
     }
   }
 
   //Show untitled explicitly:
   //Also happens for file_uris with path but no name. e.g. /sub/sub/, which shouldn't happen.
-  if(strResult.size() == 0)
+  if(strResult.empty())
     strResult = gettext("Untitled");
 
   return strResult;
@@ -301,19 +368,28 @@ bool Document::get_read_only() const
       return false; //It must be a default empty document, not yet saved, so it is not read-only.
     else
     {
+      Glib::RefPtr<Gnome::Vfs::FileInfo> info;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       try
       {
-        Gnome::Vfs::FilePermissions permissions = Gnome::Vfs::Handle::get_file_info(m_file_uri, Gnome::Vfs::FILE_INFO_GET_ACCESS_RIGHTS)->get_permissions();
-        bool read_only = ((permissions & Gnome::Vfs::PERM_ACCESS_WRITABLE) != Gnome::Vfs::PERM_ACCESS_WRITABLE);
-        return read_only;
+        info = Gnome::Vfs::Handle::get_file_info(m_file_uri, Gnome::Vfs::FILE_INFO_GET_ACCESS_RIGHTS);
       }
       catch(const Gnome::Vfs::exception& ex)
       {
         return false; //We should at least be able to read the permissions, so maybe the location is invalid. I'm not sure what the best return result here is.
       }
+#else
+      std::auto_ptr<Gnome::Vfs::exception> error;
+      info = Gnome::Vfs::Handle::get_file_info(m_file_uri, Gnome::Vfs::FILE_INFO_GET_ACCESS_RIGHTS, error);
+      if(error.get() != NULL) return false;
+#endif
+
+      Gnome::Vfs::FilePermissions permissions = info->get_permissions();
+      const bool read_only = ((permissions & Gnome::Vfs::PERM_ACCESS_WRITABLE) != Gnome::Vfs::PERM_ACCESS_WRITABLE);
+      return read_only;
     }
   }
-  
+
   return m_bReadOnly;
 }
 
@@ -331,18 +407,18 @@ void Document::set_is_new(bool bVal)
 {
   if(bVal)
     set_modified(false); //can't be modified if it is new.
-    
-  m_bIsNew = bVal; 
+
+  m_bIsNew = bVal;
 }
 
 void Document::set_file_extension(const Glib::ustring& strVal)
 {
-  m_strFileExtension = strVal;
+  m_file_extension = strVal;
 }
 
 Glib::ustring Document::get_file_extension() const
 {
-  return m_strFileExtension; //TODO: get it from the mime-type system?
+  return m_file_extension; //TODO: get it from the mime-type system?
 }
 
 Document::type_signal_modified& Document::signal_modified()
