@@ -21,6 +21,7 @@
 
 #include <gtkmm/window.h>
 #include <gdkmm/cursor.h>
+#include <map>
 
 namespace Bakery
 {
@@ -31,15 +32,29 @@ namespace Bakery
 class BusyCursor
 {
 public:
+  /** Associate a busy cursor with the window, for the lifetime of this object.
+   */
   BusyCursor(Gtk::Window& window, Gdk::CursorType cursor_type = Gdk::WATCH);
+
+  /**  Associate a busy cursor with the window, for the lifetime of this object, if window is not 0.
+   */
+  BusyCursor(Gtk::Window* window, Gdk::CursorType cursor_type = Gdk::WATCH);
+
   virtual ~BusyCursor();
 
 protected:
-    
+
+  void init();
   void force_gui_update();
 
   Gdk::Cursor m_Cursor;
+  Gtk::Window* m_pWindow;
   Glib::RefPtr<Gdk::Window> m_refWindow;
+
+  typedef std::map<Gtk::Window*, Gdk::Cursor> type_map_cursors;
+  static type_map_cursors m_map_cursors;
+  Gdk::Cursor m_old_cursor;
+  bool m_old_cursor_valid;
 };
 
 } //namespace Bakery
